# frozen_string_literal: true

require 'influxdb-client'

module Latias
  # Influxdb
  module Influxdb
    # Bucket
    class Bucket
      attr_reader :url, :token, :bucket, :org, :use_ssl, :verify_mode, :precision

      def initialize
        @query = ''
        @url = 'http://localhost:8086'
        @token = 'token'
        @bucket = 'bucket'
        @org = 'org'
        @use_ssl = false
        @verify_mode = OpenSSL::SSL::VERIFY_NONE
      end

      def call
        all_query = "from(bucket: \"#{@bucket}\") #{@query}"
        @query = ''
        @client.create_query_api.query(query: all_query, org: @org)
      end

      # old code
      # def call
      #   all_query = "from(bucket: \"#{@bucket}\")" \
      #           '|> range(start: -1m)
      #           |> filter(fn: (r) => r["_measurement"] == "cpu")
      #           |> filter(fn: (r) => r["_field"] == "usage_user")
      #           |> filter(fn: (r) => r["cpu"] == "cpu-total" or r["cpu"] == "cpu0" or r["cpu"] == "cpu1")
      #           |> yield(name: "sum")'

      #   @client.create_query_api.query(query: all_query, org: @org)
      # end

      def yield(query)
        @query += yield_format(query)
        self
      end

      def range(query)
        @query += range_format(query)
        self
      end

      def filter(query)
        @query += filter_format(query)
        self
      end

      private

      def yield_format(query)
        "#{new_command} yield(#{query})"
      end

      def range_format(query)
        "#{new_command} range(#{query})"
      end

      def filter_format(query)
        "#{new_command} filter(#{query})"
      end

      def new_command
        '|> '
      end

      def client
        @client = InfluxDB2::Client.new(@url, @token,
                                        use_ssl: @use_ssl,
                                        verify_mode: @verify_mode.present? ? @verify_mode : OpenSSL::SSL::VERIFY_NONE,
                                        precision: InfluxDB2::WritePrecision::NANOSECOND)
      end
    end
  end
end
